#!/usr/bin/env python
import os
import sys

import nibabel as nib
import numpy as np
import matplotlib.pyplot as plt
from imageio import mimsave

axis_enum = {'sagittal':0, 'coronal':1,'axial':2}

def get_slice(volume_3d, axis_name, indice=None):
    # Personal preference for orientation (nose up for axial, nose left for sagittal, nose first for coronal)
    # Operation done in 2D, still very fast and easier to understand
    if axis_name == 'sagittal':
        if indice is None:
            indice = volume_3d.shape[0] / 2
        slice_2d = np.rot90(volume_3d[indice,:,:])
    elif axis_name == 'coronal':
        if indice is None:
            indice = volume_3d.shape[1] / 2
        slice_2d = np.rot90(np.flip(volume_3d, axis=1)[:,indice,:])
    elif axis_name == 'axial':
        if indice is None:
            indice = volume_3d.shape[2] / 2
        slice_2d = np.rot90(np.flip(volume_3d, axis=2)[:,:,indice])
    else:
        raise ValueError('{0} is not a valid axis name'.format(axis))

    return slice_2d


def save_animated_slices(data, axis_name='axial', output_filename=None):
    fig = plt.figure()  
    images_list = []
    for i in range(0,data.shape[axis_enum[axis_name]]):
        images_list.append(get_slice(data, axis_name, indice=i).astype(np.uint8))

    if output_filename is None:
        output_filename = '{0}.gif'.format(axis_name)
    if os.path.isfile(output_filename):
        print(output_filename, 'already exist, overwritting')

    mimsave(output_filename, images_list, format="GIF", duration=30)

    
def median_filter(data, win_size=1):
    indices = np.nonzero(data)
    len_indices = len(np.nonzero(data)[0])
    
    new_data = np.zeros(data.shape)
    for i in range(len_indices):
        x,y,z = (indices[0][i], indices[1][i], indices[2][i])
        neighborhood = data[x-win_size:x+win_size,y-win_size:y+win_size,z-win_size:z+win_size]
        new_data[x,y,z] = np.median(neighborhood)
        
    return new_data


def inverse_image(img):
    return np.abs(img - np.max(img))


def non_lin_func_image(img):
    img_max = np.max(img)
    img_shape = img.shape
    img = img.ravel() / np.max(img)
    for i in range(len(img)):
        if np.random.rand() > 0.6: 
            img[i] = np.sin(8 * 3.141592 * img[i]) + 10
        else:
            img[i] = (4 * img[i] + 1) ** 2

    img = img.reshape(img_shape) / np.max(img)
    return img * img_max


def normalize_intensities(data, max_val):
    data -= np.min(data)
    data = (data / np.max(data)) * max_val

    return data


def add_gaussian_noise(img, mu=0, sigma=5, constrain_intensity=True, max_val=128):
    noise = np.random.normal(mu, sigma, np.prod(img.shape)).reshape(img.shape)
    noisy_img = img + noise
    
    if constrain_intensity:
        normalize_intensities(noisy_img, max_val)
    
    return noisy_img


def compare_image(img_1, img_2, filename=None):
    NBR_IMG = 3
    fig, ax_arr = plt.subplots(nrows=1, ncols=NBR_IMG)

    img_list = [img_1, img_2, img_1 - img_2]
    img_title = ['Before', 'After', 'Difference']

    for i in range(NBR_IMG):
        ax_arr[i].imshow(img_list[i], cmap='gray')
        ax_arr[i].set_title(img_title[i])
        ax_arr[i].axis('off')

    plt.tight_layout()
    if filename is None:
    	plt.show()
    else:
    	plt.savefig(filename, dpi=300)
    plt.close()


def split_name_with_nii(filename):
    base, ext = os.path.splitext(filename)

    if ext == ".gz":
        # Test if we have a .nii additional extension
        tmp_base, extra_ext = os.path.splitext(base)

        if extra_ext == ".nii":
            ext = extra_ext + ext
            base = tmp_base

    return base, ext
